// -*- C++ -*-
/*!
 * @file  DroneSensorDataConvert.h
 * @brief Sensor Data Json Convert
 * @date  $Date$
 *
 * @author East Japan Accounting Center Co.,Ltd.
 *
 * (c)2017 The University of Aizu
 * This software is released under the New BSD License.
 *
 * $Id$
 */

#ifndef DRONESENSORDATACONVERT_H
#define DRONESENSORDATACONVERT_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>
#include "VersatileOverheadSystem.h"
#include "json.h"
#include "writer.h"
#include <chrono>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

struct OutDroneSensorData_st{
	double_type	dGyro_Roll;
	double_type	dGyro_Pitch;
	double_type	dGyro_Yaw;
	float_type	fSpeed;
	float_type	fTemp;
	float_type	fCompass;
	float_type	fWindpress;
};

#define DRONE_1		0
#define DRONE_2		1
#define DRONE_3		2

/*!
 * @class DroneSensorDataConvert
 * @brief Sensor Data Json Convert
 *
 * 別紙"機能仕様書_ドローンセンサデータ変換RTC.docx"参照
 *
 * 別紙"機能仕様書_ドローンセンサデータ変換RTC.docx"参照
 *
 * 別紙"機能仕様書_ドローンセンサデータ変換RTC.docx"参照
 *
 * 別紙"機能仕様書_ドローンセンサデータ変換RTC.docx"参照
 *
 */
class DroneSensorDataConvert
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  DroneSensorDataConvert(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~DroneSensorDataConvert();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * ドローンセンサデータ変換RTCの初期化処理
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * ドローンセンサデータ変換RTCの終了処理
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * ドローンセンサデータ変換RTCのアクティブ処理
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * ドローンセンサデータ変換RTCの非アクティブ処理
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * ドローンセンサデータ変換RTCの周期処理
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   * ドローンセンサデータ変換RTCのエラー遷移処理
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * ドローンセンサデータ出力周期を調整する
   * - Name: Output_Cycle Output_Cycle
   * - DefaultValue: 0
   * - Unit: ms
   * - Range: 0<=x<=10000
   * - Constraint: -
   */
  long int m_Output_Cycle;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::TimedOrientation3D m_inDrone1GyroData;
  /*!
   * ドローン1のGyroデータを取得するデータポート
   * - Type: RTC::TimedOrientation3D
   * - Number: 1
   */
  InPort<RTC::TimedOrientation3D> m_inDrone1GyroDataIn;
  RTC::TimedFloat m_inDrone1SpeedData;
  /*!
   * ドローン1のSpeedデータを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone1SpeedDataIn;
  RTC::TimedFloat m_inDrone1TempData;
  /*!
   * ドローン1の温度データを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone1TempDataIn;
  RTC::TimedFloat m_inDrone1CompassData;
  /*!
   * ドローン1のコンパスデータを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone1CompassDataIn;
  RTC::TimedFloat m_inDrone1WindPressData;
  /*!
   * ドローン1の風圧データを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone1WindPressDataIn;
  RTC::TimedOrientation3D m_inDrone2GyroData;
  /*!
   * ドローン2のGyroデータを取得するデータポート
   * - Type: RTC::TimedOrientation3D
   * - Number: 1
   */
  InPort<RTC::TimedOrientation3D> m_inDrone2GyroDataIn;
  RTC::TimedFloat m_inDrone2SpeedData;
  /*!
   * ドローン2のSpeedデータを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone2SpeedDataIn;
  RTC::TimedFloat m_inDrone2TempData;
  /*!
   * ドローン2の温度データを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone2TempDataIn;
  RTC::TimedFloat m_inDrone2CompassData;
  /*!
   * ドローン2のコンパスデータを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone2CompassDataIn;
  RTC::TimedFloat m_inDrone2WindPressData;
  /*!
   * ドローン2の風圧データを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone2WindPressDataIn;
  RTC::TimedOrientation3D m_inDrone3GyroData;
  /*!
   * ドローン3のGyroデータを取得するデータポート
   * - Type: RTC::TimedOrientation3D
   * - Number: 1
   */
  InPort<RTC::TimedOrientation3D> m_inDrone3GyroDataIn;
  RTC::TimedFloat m_inDrone3SpeedData;
  /*!
   * ドローン3のSpeedデータを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone3SpeedDataIn;
  RTC::TimedFloat m_inDrone3TempData;
  /*!
   * ドローン3の温度データを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone3TempDataIn;
  RTC::TimedFloat m_inDrone3CompassData;
  /*!
   * ドローン3のコンパスデータを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone3CompassDataIn;
  RTC::TimedFloat m_inDrone3WindPressData;
  /*!
   * ドローン3の風圧データを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inDrone3WindPressDataIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::TimedStringSeq m_outDroneSensorData;
  /*!
   * ドローンのセンサデータをJson形式で台数分提供するデータポート
   * - Type: RTC::TimedStringSeq
   * - Number: 3
   * - Frequency: 500ms周期
   * - Operation Cycle: 500ms周期
   */
  OutPort<RTC::TimedStringSeq> m_outDroneSensorDataOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

	 void nvdDroneDataRamInit(void);
	 bool_type blDroneDataPortRead(void);
	 uhword_type uhwDroneSensorDataOutput(void);

	 InPort<RTC::TimedOrientation3D> *m_InGyroDataIn_p[DRONE_CONNECT_MAX];
	 InPort<RTC::TimedFloat> *m_InSpeedDataIn_p[DRONE_CONNECT_MAX];
	 InPort<RTC::TimedFloat> *m_InTempDataIn_p[DRONE_CONNECT_MAX];
	 InPort<RTC::TimedFloat> *m_InCompassDataIn_p[DRONE_CONNECT_MAX];
	 InPort<RTC::TimedFloat> *m_InWindpressDataIn_p[DRONE_CONNECT_MAX];

	 RTC::TimedOrientation3D *m_InGyroData_p[DRONE_CONNECT_MAX];
	 RTC::TimedFloat *m_InSpeedData_p[DRONE_CONNECT_MAX];
	 RTC::TimedFloat *m_InTempData_p[DRONE_CONNECT_MAX];
	 RTC::TimedFloat *m_InCompassData_p[DRONE_CONNECT_MAX];
	 RTC::TimedFloat *m_InWindpressData_p[DRONE_CONNECT_MAX];

	 OutDroneSensorData_st m_stDroneData[DRONE_CONNECT_MAX];

	 std::chrono::system_clock::time_point  m_LastTime;	//前回のセンサデータ出力時間を保持
};


extern "C"
{
  DLL_EXPORT void DroneSensorDataConvertInit(RTC::Manager* manager);
};

#endif // DRONESENSORDATACONVERT_H
