// -*- C++ -*-
/*!
 * @file  DroneSensorDataConvert.cpp
 * @brief Sensor Data Json Convert
 * @date $Date$
 *
 * @author East Japan Accounting Center Co.,Ltd.
 *
 * (c)2017 The University of Aizu
 * This software is released under the New BSD License.
 *
 * $Id$
 */

#include "DroneSensorDataConvert.h"

// Module specification
// <rtc-template block="module_spec">
static const char* dronesensordataconvert_spec[] =
  {
    "implementation_id", "DroneSensorDataConvert",
    "type_name",         "DroneSensorDataConvert",
    "description",       "Sensor Data Json Convert",
    "version",           "0.1",
    "vendor",            "EAC",
    "category",          "Sensor",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Output_Cycle", "0",
    // Widget
    "conf.__widget__.Output_Cycle", "spin.500",
    // Constraints
    "conf.__constraints__.Output_Cycle", "0<=x<=10000",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
DroneSensorDataConvert::DroneSensorDataConvert(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inDrone1GyroDataIn("inDrone1GyroData", m_inDrone1GyroData),
    m_inDrone1SpeedDataIn("inDrone1SpeedData", m_inDrone1SpeedData),
    m_inDrone1TempDataIn("inDrone1TempData", m_inDrone1TempData),
    m_inDrone1CompassDataIn("inDrone1CompassData", m_inDrone1CompassData),
    m_inDrone1WindPressDataIn("inDrone1WindPressData", m_inDrone1WindPressData),
    m_inDrone2GyroDataIn("inDrone2GyroData", m_inDrone2GyroData),
    m_inDrone2SpeedDataIn("inDrone2SpeedData", m_inDrone2SpeedData),
    m_inDrone2TempDataIn("inDrone2TempData", m_inDrone2TempData),
    m_inDrone2CompassDataIn("inDrone2CompassData", m_inDrone2CompassData),
    m_inDrone2WindPressDataIn("inDrone2WindPressData", m_inDrone2WindPressData),
    m_inDrone3GyroDataIn("inDrone3GyroData", m_inDrone3GyroData),
    m_inDrone3SpeedDataIn("inDrone3SpeedData", m_inDrone3SpeedData),
    m_inDrone3TempDataIn("inDrone3TempData", m_inDrone3TempData),
    m_inDrone3CompassDataIn("inDrone3CompassData", m_inDrone3CompassData),
    m_inDrone3WindPressDataIn("inDrone3WindPressData", m_inDrone3WindPressData),
    m_outDroneSensorDataOut("outDroneSensorData", m_outDroneSensorData)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
DroneSensorDataConvert::~DroneSensorDataConvert()
{
}


/*!
 * ドローンセンサデータ変換RTCの初期化処理
 */
RTC::ReturnCode_t DroneSensorDataConvert::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("inDrone1GyroData", m_inDrone1GyroDataIn);
  addInPort("inDrone1SpeedData", m_inDrone1SpeedDataIn);
  addInPort("inDrone1TempData", m_inDrone1TempDataIn);
  addInPort("inDrone1CompassData", m_inDrone1CompassDataIn);
  addInPort("inDrone1WindPressData", m_inDrone1WindPressDataIn);
  addInPort("inDrone2GyroData", m_inDrone2GyroDataIn);
  addInPort("inDrone2SpeedData", m_inDrone2SpeedDataIn);
  addInPort("inDrone2TempData", m_inDrone2TempDataIn);
  addInPort("inDrone2CompassData", m_inDrone2CompassDataIn);
  addInPort("inDrone2WindPressData", m_inDrone2WindPressDataIn);
  addInPort("inDrone3GyroData", m_inDrone3GyroDataIn);
  addInPort("inDrone3SpeedData", m_inDrone3SpeedDataIn);
  addInPort("inDrone3TempData", m_inDrone3TempDataIn);
  addInPort("inDrone3CompassData", m_inDrone3CompassDataIn);
  addInPort("inDrone3WindPressData", m_inDrone3WindPressDataIn);
  
  // Set OutPort buffer
  addOutPort("outDroneSensorData", m_outDroneSensorDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Output_Cycle", m_Output_Cycle, "0");
  // </rtc-template>
 
#ifdef _DEBUG
  std::cout << "[info]DroneSensorDataConvert::onInitialize()" << std::endl;
#endif

  nvdDroneDataRamInit();
  m_outDroneSensorData.data.length(DRONE_CONNECT_MAX);

  m_InGyroDataIn_p[DRONE_1] = &m_inDrone1GyroDataIn;
  m_InGyroDataIn_p[DRONE_2] = &m_inDrone2GyroDataIn;
  m_InGyroDataIn_p[DRONE_3] = &m_inDrone3GyroDataIn;
  m_InGyroData_p[DRONE_1] = &m_inDrone1GyroData;
  m_InGyroData_p[DRONE_2] = &m_inDrone2GyroData;
  m_InGyroData_p[DRONE_3] = &m_inDrone3GyroData;

  m_InSpeedDataIn_p[DRONE_1] = &m_inDrone1SpeedDataIn;
  m_InSpeedDataIn_p[DRONE_2] = &m_inDrone2SpeedDataIn;
  m_InSpeedDataIn_p[DRONE_3] = &m_inDrone3SpeedDataIn;
  m_InSpeedData_p[DRONE_1] = &m_inDrone1SpeedData;
  m_InSpeedData_p[DRONE_2] = &m_inDrone2SpeedData;
  m_InSpeedData_p[DRONE_3] = &m_inDrone3SpeedData;

  m_InTempDataIn_p[DRONE_1] = &m_inDrone1TempDataIn;
  m_InTempDataIn_p[DRONE_2] = &m_inDrone2TempDataIn;
  m_InTempDataIn_p[DRONE_3] = &m_inDrone3TempDataIn;
  m_InTempData_p[DRONE_1] = &m_inDrone1TempData;
  m_InTempData_p[DRONE_2] = &m_inDrone2TempData;
  m_InTempData_p[DRONE_3] = &m_inDrone3TempData;

  m_InCompassDataIn_p[DRONE_1] = &m_inDrone1CompassDataIn;
  m_InCompassDataIn_p[DRONE_2] = &m_inDrone2CompassDataIn;
  m_InCompassDataIn_p[DRONE_3] = &m_inDrone3CompassDataIn;
  m_InCompassData_p[DRONE_1] = &m_inDrone1CompassData;
  m_InCompassData_p[DRONE_2] = &m_inDrone2CompassData;
  m_InCompassData_p[DRONE_3] = &m_inDrone3CompassData;

  m_InWindpressDataIn_p[DRONE_1] = &m_inDrone1WindPressDataIn;
  m_InWindpressDataIn_p[DRONE_2] = &m_inDrone2WindPressDataIn;
  m_InWindpressDataIn_p[DRONE_3] = &m_inDrone3WindPressDataIn;
  m_InWindpressData_p[DRONE_1] = &m_inDrone1WindPressData;
  m_InWindpressData_p[DRONE_2] = &m_inDrone2WindPressData;
  m_InWindpressData_p[DRONE_3] = &m_inDrone3WindPressData;

  return RTC::RTC_OK;
}

/*!
 * ドローンセンサデータ変換RTCの終了処理
 */

RTC::ReturnCode_t DroneSensorDataConvert::onFinalize()
{
#ifdef _DEBUG
	std::cout << "[info]DroneSensorDataConvert::onFinalize()" << std::endl;
#endif

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DroneSensorDataConvert::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DroneSensorDataConvert::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * ドローンセンサデータ変換RTCのアクティブ処理
 */

RTC::ReturnCode_t DroneSensorDataConvert::onActivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[info]DroneSensorDataConvert::onActivated()" << std::endl;
#endif

	//入力データポートの初期化
	while(m_inDrone1GyroDataIn.isEmpty() == FALSE){
		m_inDrone1GyroDataIn.read();
	}
	while(m_inDrone1SpeedDataIn.isEmpty() == FALSE){
		m_inDrone1SpeedDataIn.read();
	}
	while(m_inDrone1TempDataIn.isEmpty() == FALSE){
		m_inDrone1TempDataIn.read();
	}
	while(m_inDrone1CompassDataIn.isEmpty() == FALSE){
		m_inDrone1CompassDataIn.read();
	}
	while(m_inDrone1WindPressDataIn.isEmpty() == FALSE){
		m_inDrone1WindPressDataIn.read();
	}
	while(m_inDrone2GyroDataIn.isEmpty() == FALSE){
		m_inDrone2GyroDataIn.read();
	}
	while(m_inDrone2SpeedDataIn.isEmpty() == FALSE){
		m_inDrone2SpeedDataIn.read();
	}
	while(m_inDrone2TempDataIn.isEmpty() == FALSE){
		m_inDrone2TempDataIn.read();
	}
	while(m_inDrone2CompassDataIn.isEmpty() == FALSE){
		m_inDrone2CompassDataIn.read();
	}
	while(m_inDrone2WindPressDataIn.isEmpty() == FALSE){
		m_inDrone2WindPressDataIn.read();
	}
	while(m_inDrone3GyroDataIn.isEmpty() == FALSE){
		m_inDrone3GyroDataIn.read();
	}
	while(m_inDrone3SpeedDataIn.isEmpty() == FALSE){
		m_inDrone3SpeedDataIn.read();
	}
	while(m_inDrone3TempDataIn.isEmpty() == FALSE){
		m_inDrone3TempDataIn.read();
	}
	while(m_inDrone3CompassDataIn.isEmpty() == FALSE){
		m_inDrone3CompassDataIn.read();
	}
	while(m_inDrone3WindPressDataIn.isEmpty() == FALSE){
		m_inDrone3WindPressDataIn.read();
	}

	//経過時間保持変数を初期化
	m_LastTime = std::chrono::system_clock::now();

	return RTC::RTC_OK;
}

/*!
 * ドローンセンサデータ変換RTCの非アクティブ処理
 */

RTC::ReturnCode_t DroneSensorDataConvert::onDeactivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[info]DroneSensorDataConvert::onDeactivated()" << std::endl;
#endif

	nvdDroneDataRamInit();

	return RTC::RTC_OK;
}

/*!
 * ドローンセンサデータ変換RTCの周期処理
 */

RTC::ReturnCode_t DroneSensorDataConvert::onExecute(RTC::UniqueId ec_id)
{
	bool_type blRet = FALSE;
	std::chrono::system_clock::time_point  CurrentTime;
	double dElapsedTime = 0;

	CurrentTime = std::chrono::system_clock::now();	//現在の時間を取得
	dElapsedTime = std::chrono::duration_cast<std::chrono::milliseconds>(CurrentTime - m_LastTime).count();	//経過時間をミリ秒に変換

	//コンフィギュレーション出力周期経過後、GPSデータ出力
	if(dElapsedTime >= m_Output_Cycle){
		blRet = blDroneDataPortRead();
		if(blRet){
			uhwDroneSensorDataOutput();
		}
		m_LastTime = CurrentTime;
	}

	return RTC::RTC_OK;
}

/*!
 * ドローンセンサデータ変換RTCのエラー遷移処理
 */

RTC::ReturnCode_t DroneSensorDataConvert::onAborting(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[info]DroneSensorDataConvert::onAborting()" << std::endl;
#endif

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DroneSensorDataConvert::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DroneSensorDataConvert::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DroneSensorDataConvert::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DroneSensorDataConvert::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*****************************************************************************************
*	Function Name		：blDroneDataPortRead
*	Description			：全てのドローンデータポート読み込み
*	Argument			：なし
*	Return				：TRUE：更新あり、FALSE：更新なし
******************************************************************************************/
bool_type DroneSensorDataConvert::blDroneDataPortRead(void)
{
	bool_type blRet = FALSE;
	uhword_type uhwLoop = DRONE_1;

	for(uhwLoop = DRONE_1; uhwLoop < DRONE_CONNECT_MAX; uhwLoop ++){
		if(m_InGyroDataIn_p[uhwLoop]->isNew()){
			m_InGyroDataIn_p[uhwLoop]->read();
			m_stDroneData[uhwLoop].dGyro_Roll = m_InGyroData_p[uhwLoop]->data.r;
			m_stDroneData[uhwLoop].dGyro_Pitch = m_InGyroData_p[uhwLoop]->data.p;
			m_stDroneData[uhwLoop].dGyro_Yaw = m_InGyroData_p[uhwLoop]->data.y;
			blRet = TRUE;
		}

		if(m_InSpeedDataIn_p[uhwLoop]->isNew()){
			m_InSpeedDataIn_p[uhwLoop]->read();
			m_stDroneData[uhwLoop].fSpeed = m_InSpeedData_p[uhwLoop]->data;
			blRet = TRUE;
		}

		if(m_InTempDataIn_p[uhwLoop]->isNew()){
			m_InTempDataIn_p[uhwLoop]->read();
			m_stDroneData[uhwLoop].fTemp = m_InTempData_p[uhwLoop]->data;
			blRet = TRUE;
		}

		if(m_InCompassDataIn_p[uhwLoop]->isNew()){
			m_InCompassDataIn_p[uhwLoop]->read();
			m_stDroneData[uhwLoop].fCompass = m_InCompassData_p[uhwLoop]->data;
			blRet = TRUE;
		}

		if(m_InWindpressDataIn_p[uhwLoop]->isNew()){
			m_InWindpressDataIn_p[uhwLoop]->read();
			m_stDroneData[uhwLoop].fWindpress = m_InWindpressData_p[uhwLoop]->data;
			blRet = TRUE;
		}
	}

	return 	blRet;
}

/*****************************************************************************************
*	Function Name		：uhwDroneSensorDataOutput
*	Description			：ドローンセンサデータ出力
*	Argument			：なし
*	Return				：正常終了
******************************************************************************************/
uhword_type DroneSensorDataConvert::uhwDroneSensorDataOutput(void)
{
	Json::StyledWriter	writer;
	std::string strJson = "";
	uhword_type uhwLoop = DRONE_1;
	Json::Value JsonData;

	for(uhwLoop = DRONE_1; uhwLoop < DRONE_CONNECT_MAX; uhwLoop++){
		JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_ROLL] = m_stDroneData[uhwLoop].dGyro_Roll;
		JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_PITCH] = m_stDroneData[uhwLoop].dGyro_Pitch;
		JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_YAW] = m_stDroneData[uhwLoop].dGyro_Yaw;
		JsonData[JSONKEY_SENSOR_SPEED] = m_stDroneData[uhwLoop].fSpeed;
		JsonData[JSONKEY_SENSOR_TEMP] = m_stDroneData[uhwLoop].fTemp;
		JsonData[JSONKEY_SENSOR_COMPASS] = m_stDroneData[uhwLoop].fCompass;
		JsonData[JSONKEY_SENSOR_WINDPRESS] = m_stDroneData[uhwLoop].fWindpress;

		//jsonフォーマットに変換し出力
		strJson = writer.write(JsonData);
		m_outDroneSensorData.data[uhwLoop] = strJson.c_str();
		std::cout << "[debug]Drone No = " << uhwLoop << ",Output = " << strJson << std::endl;
	}
	m_outDroneSensorDataOut.write();

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：nvdDroneDataRamInit
*	Description			：メンバ変数初期化
*	Argument			：なし
*	Return				：なし
******************************************************************************************/
void DroneSensorDataConvert::nvdDroneDataRamInit(void)
{
	memset(m_stDroneData, 0, sizeof(m_stDroneData));
}

extern "C"
{
 
  void DroneSensorDataConvertInit(RTC::Manager* manager)
  {
    coil::Properties profile(dronesensordataconvert_spec);
    manager->registerFactory(profile,
                             RTC::Create<DroneSensorDataConvert>,
                             RTC::Delete<DroneSensorDataConvert>);
  }
  
};


